/* FreeVR interface
 * Copyright (C) 2001 Paul Rajlich
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <freevr.h>
#include <GL/gl.h>
#include "vec.h"
#include "uicommon.h"
#include "rendercontext.h"
#include "globalshared.h"

global_shared_t *g;
static r_context_t *r_context;
static float *time;

/* use shared memory for global shared data */
void *gc_malloc(size_t size) { return vrShmemAlloc0(size); }
void *rc_malloc(size_t size) { return malloc(size); }

static void renderProcInit();
static void display();
static void update();

int
main(int argc, char **argv)
{
    /* defaults */
    char *pak = "pak0.pk3", *bsp = "maps/q3tourney2.bsp";

    /* default shared mem arena is too small! */
    vrShmemInit(64*1024*1024);
    vrConfigure(&argc, argv, NULL);
    /* CAVEFar = 1000.0f; */

    /* setup shared data structures */
    g = (global_shared_t *) gc_malloc(sizeof(global_shared_t));
    init_global_shared(g);
    ui_read_args(argc, argv, &bsp, &pak);

    /* load the map */
    ui_init_bsp(bsp, pak);
    printf("\n\n");

    /* FreeVR will setup projection(s) */
    g->r_setup_projection = 0;

    time = (float *) gc_malloc(sizeof(float));
    *time = 0;

    vrStart();
    vrSetFunc(VRFUNC_ALL_DISPLAY_INIT, vrCreateCallback(renderProcInit, 0));
    vrSetFunc(VRFUNC_ALL_DISPLAY, vrCreateCallback(display, 0));

    while (!vrGetSwitch2Value(0)) {
      update();
    }
    vrExit();

    /* ui_finish(); */
    return 0;
}

static void
renderProcInit()
{
  r_context = (r_context_t *) rc_malloc(sizeof(r_context_t));
  ui_init_gl(r_context);
}

static void
display(void)
{
  ui_display(r_context, *time);
  glDisable(GL_BLEND);
}

static void
update()
{
  static float lastTime = 0.0f;
  float etime;
  float wandO[3];
  vec3_t dir;

  *time = vrNow();
  etime = *time - lastTime;

  if (etime > 0.2f)
    etime = 0.2f; /* cap */

  /* rotate CAVE */
  if (fabsf(vrGetValuatorValue(0)) > 0.3f)
    g->r_eye_az = g->r_eye_az - vrGetValuatorValue(0)*etime*50.0f;

  /* move forward/backward */
  if (vrGetValuatorValue(1) > 0.5f)       ui_key_down('w');
  else if (vrGetValuatorValue(1) < -0.5f) ui_key_down('s');
  else if (vrGetSwitch2Value(1))          ui_key_down('w');
  else
    ui_key_up('w');



  /* change sky */
  if (vrGetSwitch2Delta(3) == 1)
    ui_key_down('b');

  /* turn on/off collision detection */
  if (vrGetSwitch2Delta(2) == 1)
    ui_key_down('c');

  /* travel in direction wand is pointing - skip for now */
  /* CAVEGetOrientation(CAVE_WAND, wandO); */
  wandO[1] = 0.0f;  wandO[2] = 0.0f;
  vec_point(dir, wandO[1]+g->r_eye_az, wandO[0]); /* world coords */

  g->r_movedir[0] = dir[0];
  g->r_movedir[1] = dir[1];
  g->r_movedir[2] = dir[2];

  ui_move(*time);

  lastTime = *time;
}
